


class Filter
{
  // This class is for filtering/smoothing
  // It provides 3 options.
  // 1) Exponential Moving Average (EMA)
  // 2) Double Exponential Moving Average (DEMA)
  // 3) Triple Exponential Moving Average (TEMA)
  //
  // Choose only use one of the functions.
  //
  // coeff [0..1] if near to zero filter more inert
  // e.g. 0.05 powerful filter  0.9 light filtering
  //  
  private:
  float ema = 0;
  float ema_ema = 0;
  float ema_ema_ema = 0;
  int   usingfilter = 1; // 1 = EMA, 2 = DEMA, 3 = TEMA
  const float _coeff;
  
  public:

  Filter(float coeff):
  _coeff(coeff)
  {
  }
  
  // set initial value
  void setValue(float value)
  {
    ema = value;
    ema_ema = value;
    ema_ema_ema = value;
  }
  
  //get filtered value
  float getValue() const
  {
    if (usingfilter == 1)
      return ema;
    else 
      if (usingfilter == 2)
        return ema_ema;
      else
        return ema_ema_ema;
  }

  float EMA_function( float latest, float stored)
  {
    return (_coeff*latest) + ((1-_coeff)*stored);
  }

  float EMA(float value)
  {
    // Use this for exponential moving average
    ema = EMA_function( value, ema);
    usingfilter = 1;
    return ema;
  }

  float DEMA(float value)
  {
    // Use this for double exponential moving average    
    ema = EMA_function( value, ema);
    ema_ema = EMA_function( ema, ema_ema); 
    usingfilter = 2;
    return 2 * ema - ema_ema;
  }

  float TEMA(float value)
  {
    // Use this for Triple exponential moving average
    ema = EMA_function( value, ema);
    ema_ema = EMA_function( ema, ema_ema); 
    ema_ema_ema = EMA_function( ema_ema, ema_ema_ema);
    usingfilter = 3;
    return 3*ema-3*ema_ema+ema_ema_ema;
  }  
};
